from e2enetworks.cloud.tir import version as tir_version

__version__ = tir_version.__version__
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.pipelines import PipelineClient
from e2enetworks.cloud.tir.images import Images
from e2enetworks.cloud.tir.skus import Plans
from e2enetworks.cloud.tir.apitoken import APITokens
from e2enetworks.cloud.tir.notebook import Notebooks
from e2enetworks.cloud.tir.datasets import Datasets
from e2enetworks.cloud.tir.models import Models
from e2enetworks.cloud.tir.endpoints import EndPoints
from e2enetworks.cloud.tir.projects import Projects
from e2enetworks.cloud.tir.teams import Teams


init = client.Default.init

__all__ = (
    "init",
    "PipelineClient"
)


def help():
    print("\t\tAIPlatform Help")
    print("\t\t=================")
    print("\t\tAvailable classes:")
    print("\t\t- init: Provides functionalities for initialization.")
    print("\t\t- Images: Provides functionalities to interact with images.")
    print("\t\t- Plans: Provides functionalities to interact with skus.")
    print("\t\t- Teams: Provides functionalities to interact with teams.")
    print("\t\t- Projects: Provides functionalities to interact with projects.")
    print("\t\t- Notebooks: Provides functionalities to interact with notebooks.")
    print("\t\t- Datasets: Provides functionalities to interact with datasets.")
    print("\t\t- EndPoints: Provides functionalities to interact with endpoints.")
    print("\t\t- Models: Provides functionalities to interact with models.")
    print("\t\t- PipelineClient: Provides functionalities to interact with Pipelines.")
    print("\t\t- APITokens: Provides functionalities to interact with API tokens.")

    # Call help() method on each class
    client.Default.help()
    Images.help()
    Plans.help()
    Teams.help()
    Projects.help()
    Notebooks.help()
    Datasets.help()
    EndPoints.help()
    PipelineClient.help()
    Models.help()
    APITokens.help()



import requests

from typing import Optional
from e2enetworks.constants import BASE_GPU_URL
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.utils import prepare_object


class Projects:
    def __init__(
            self,
            team: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if team:
            client.Default.set_team(team)

    def create(self, project_name):
        payload = {
            "project_name": project_name,
        }
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/"
        req = requests.Request('POST', url, data=payload)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get(self, project_id):

        if type(project_id) != int:
            raise ValueError(project_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{project_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, project_id):
        if type(project_id) != int:
            raise ValueError(project_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{project_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("Projects Class Help")
        print("\t\t===================")
        print("\t\tThis class provides functionalities to interact with projects.")
        print("\t\tAvailable methods:")
        print("\t\t1. __init__(team_id): Initializes a Projects instance with the specified team ID.")
        print("\t\t2. create(project_name): Creates a new project with the provided project name.")
        print("\t\t3. get(project_id): Retrieves information about a specific project using its ID.")
        print("\t\t4. list(): Lists all projects associated with the team.")
        print("\t\t5. delete(project_id): Deletes a project with the given ID.")
        print("\t\t8. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tprojects = Projects(123)")
        print("\t\tprojects.create('Project Name')")
        print("\t\tprojects.get(456)")
        print("\t\tprojects.list()")
        print("\t\tprojects.delete(456)")



import requests

from e2enetworks.cloud.tir import client
from e2enetworks.constants import BASE_GPU_URL

id_to_hash_code_mapping = {
    1: "QHeMSAyC",
    2: "skqbtZpd",
    3: "NynWPeEJ",
    4: "XEFKbStA",
    5: "QPSKEFKQ",
    6: "siGeSJyO",
    7: "yYkVtTsC",
    8: "wVOqCnpB",
    9: "SICGgpoZ",
    10: "ddUybAyh",
    11: "yWNKwMlg",
    12: "WtZaGkNZ"
}


class Images:
    def __init__(self):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

    def list(self):
        url = f"{BASE_GPU_URL}gpu_service/image/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        if response.status_code == 200:
            images = response.json()["data"]
            print("Code               | Name")
            for image in images:
                print(f"{id_to_hash_code_mapping.get(image.get('id'))}           |{image.get('name')} {image.get('version')}")


    @staticmethod
    def help():
        print("Images Class Help")
        print("\t\t================")
        print("\t\tThis class provides functionalities to interact with Images.")
        print("\t\tAvailable methods:")

        print("\t\t1. list(): Lists all Images.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\timages = Images()")
        print("\t\timages.list()")



import yaml
import json
from typing import Dict, Any
from requests import Response
from kfp_server_api import ApiListExperimentsResponse, ApiExperiment
from types import SimpleNamespace

def load_yaml(
    path: str,
) -> Dict[str, Any]:
    _load_from_local(path)

def _load_from_local(path: str) -> Dict[str, Any]:
    with open(path) as f:
        return yaml.safe_load(f)
    

def load_json(b: bytes, object_hook=None) -> Dict:
    return json.loads(b, object_hook=object_hook)

def prepare_object(response: Response) -> ApiListExperimentsResponse:
    if not response.ok:
        try:
            output = response.json()
        except:
            output = response.reason
        print("failed to load response:", output)
        return None
    
    return load_json(response.content, object_hook=lambda d: SimpleNamespace(**d))

def prepare_response(response: Response) -> Dict:
    if response.ok:
        return load_json(response.content)
    
    # logger.error("")
    print("api call failed with error:", response.reason)
    return {
        "error": response.reason
    }


import os
import kfp
from e2enetworks.cloud.tir import client
from typing import Optional
from requests import Request, Session, Response
import e2enetworks.constants as constants

E2E_UI_HOST = "localhost:3000"

class _Client:
    def __init__(self):
        self._api_key = None
        self._access_token = None
        self._project = None
        self._team = None
        self._namespace = None
        self._eos_bucket = None
        self._api_host = None
        self._client = None

    def init(
        self,
        *,
        api_key: Optional[str] = None,
        access_token: Optional[str] = None,
        team: Optional[str] = None,
        project: Optional[str] = None,
        eos_bucket: Optional[str] = None,
    ):
        self._access_token = access_token
        self._api_key = api_key
        self._project = project
        self._team = team
        self._eos_bucket = eos_bucket

    def ready(self): 
        return False if not self.api_key() or not self.access_token() else True
    
    def api_host(self):
        if self._api_host:
            return self._api_host
        if os.environ.get("E2E_TIR_API_HOST"):
            self._api_host = os.environ.get("E2E_TIR_API_HOST")
            return self._api_host
        return constants.MY_ACCOUNT_LB_URL
    
    def access_token(self):
        if self._access_token:
            return self._access_token

        access_token_not_set = (
            "Access Token not set. Please provide a Access Token by:"
            "\n- Using e2enetworks.cloud.tir.init(access_token=)"
            "\n- Setting an environment variable E2E_TIR_ACCESS_TOKEN"
        )

        if os.environ.get("E2E_TIR_ACCESS_TOKEN"):
            self._access_token = os.environ.get("E2E_TIR_ACCESS_TOKEN")
            return self._access_token
        else:
            raise ValueError(access_token_not_set)

    def api_key(self):
        if self._api_key:
            return self._api_key

        api_key_not_set = (
            "API key not set. Please provide api key by:"
            "\n- Using e2enetworks.cloud.tir.init(api_key=)"
            "\n- Setting an environment variable E2E_TIR_API_KEY"
        )

        if os.environ.get("E2E_TIR_API_KEY"):
            self._api_key = os.environ.get("E2E_TIR_API_KEY")
            return self._api_key
        else:
            raise ValueError(api_key_not_set)

    def set_project(self, project):
        self._project = project

    def set_team(self, team):
        self._team = team

    def gpu_projects_path(self, project=None):
        if not project:
            project = self.project()
        
        return "{api_host}/api/v1/gpu/projects/{project}".format(api_host=self.api_host(), project=project)
    
    def project(self):
        if self._project:
            return self._project
        
        project_not_set = (
            "Project ID not set. Please provide a project ID by:"
            "\n- Using e2enetworks.cloud.tir.init(project=)"
            "\n- Setting an environment variable E2E_TIR_PROJECT_ID"
        )

        if os.environ.get("E2E_TIR_PROJECT_ID"):
            self._project = os.environ.get("E2E_TIR_PROJECT_ID")
            return self._project
        else:
            raise ValueError(project_not_set)

    def namespace(self, project=None):
        if not project:
            project = self.project()

        return "p-{}".format(project)

    def team(self):
        if self._team:
            return self._team
        
        team_not_set = (
            "Team ID not set. Please provide a team ID by:"
            "\n- Using e2enetworks.cloud.tir.init()"
            "\n- Setting an environment variable E2E_TIR_TEAM_ID"
        )

        if os.environ.get("E2E_TIR_TEAM_ID"):
            self._team = os.environ.get("E2E_TIR_TEAM_ID")
            return self._team
        else:
            raise ValueError(team_not_set)
    
    def make_request(self, request, stream=None, verify=None, timeout=None) -> Response:
        '''
            make_request(self, request): is common method to prepare a request
            and send to API host. 
            
            input:
            - request: requests.request. the request url must be only the path and 
            should not contain api host. 
            example usage: 
                request = requests.Request("GET", "http://localhost/api/v1/pipelines")
                make_request(request)

            output:
            - http response
            
            The modifications made to the request will be:
            * include auth header and add api key query param
            * append api host to the path 
        '''    
        s = Session()
        prepped = s.prepare_request(request)

        # append api key. we expect caller to send & if query parameters are used.
        # for example:
        #   if the request url has query parameters like below
        #       /pipelines?page=2 
        #   then we expect the caller to set url as /pipelines?page=2& instead of /pipelines?page=2
        # 
        prepped.url = f"{prepped.url}?" if prepped.url[len(prepped.url)-1] == "/" else prepped.url
        prepped.url = "{}apikey={}".format(prepped.url, self.api_key())
        print("prepped.url:", prepped.url)
        prepped.headers["Authorization"] = f"Bearer {self.access_token()}"

        return s.send(prepped, stream=stream, verify=verify, timeout=timeout)

    def print_values(self):
        print(f"apikey - {self.api_key()}")
        print(f"access_token - {self.access_token()}")
        print(f"team - {self.team()}")
        print(f"project - {self.project()}")

# common client set by tir.init(access_token=..., api_key=...)
    @staticmethod
    def help():
        help_text = '''
_Client - E2E Networks TIR Client

This class provides a client for interacting with the E2E Networks TIR API.

        Methods:

        1. init(api_key, access_token, project=None, team=None, eos_bucket=None)
           - Initialize the client with API key and access token.
           - Parameters:
             - api_key (str): Your API key.
             - access_token (str): Your access token.
             - team (str, optional): Team ID. Defaults to None.
             - project (str, optional): Project ID. Defaults to None.
             - eos_bucket (str, optional): EOS bucket. Defaults to None.

        2. ready()
           - Check if the client is ready for API requests.
           - Returns:
             - bool: True if both API key and access token are set, otherwise False.

        3. api_host()
           - Get the API host URL.
           - Returns:
             - str: The API host URL.

        4. access_token()
           - Get the access token. It can be set using `init` or an environment variable.
           - Returns:
             - str: The access token.

        5. api_key()
           - Get the API key. It can be set using `init` or an environment variable.
           - Returns:
             - str: The API key.

        6. set_project(project)
           - Set the project ID.
           - Parameters:
             - project (str): Project ID.

        7. set_team(team)
           - Set the team ID.
           - Parameters:
             - team (str): Team ID.

        8. gpu_projects_path(project=None)
           - Get the URL for GPU projects, optionally specifying a project.
           - Parameters:
             - project (str, optional): Project ID. Defaults to None.
           - Returns:
             - str: The GPU projects URL.

        9. project()
           - Get the project ID. It can be set using `init` or an environment variable.
           - Returns:
             - str: The project ID.

        10. namespace(project=None)
            - Generate a namespace for a project.
            - Parameters:
              - project (str, optional): Project ID. Defaults to None.
            - Returns:
              - str: The generated namespace.

        11. team()
            - Get the team ID. It can be set using `init` or an environment variable.
            - Returns:
              - str: The team ID.
              
        12. print_values()
            - Print values of team, project, access_token, apikey

        12. make_request(request, stream=None, verify=None, timeout=None)
            - Prepare and send a request to the API host.
            - Parameters:
              - request (Request): The request to send.
              - stream (bool, optional): Stream the response. Defaults to None.
              - verify (bool, optional): Verify SSL certificates. Defaults to None.
              - timeout (float, optional): Request timeout. Defaults to None.
            - Returns:
              - Response: The HTTP response.

        Attributes:

        Default
           - A default client instance that can be used for common client interactions.

        Usage:
        ```
        # Initialize the client
        client = _Client()
        client.init(api_key="your_api_key", access_token="your_access_token")

        # Check if the client is ready
        if client.ready():
            print("Client is ready for API requests.")
        else:
            print("Client is not ready.")

        # Access other client methods and attributes as needed.
        '''
        print(help_text)

    # Run the help function to display the documentation


Default = _Client()



MY_ACCOUNT_LB_URL = "https://api.e2enetworks.com/myaccount/"
GPU_URL = "api/v1/gpu/"
BASE_GPU_URL = f"{MY_ACCOUNT_LB_URL}{GPU_URL}"
VALIDATED_SUCCESSFULLY = "Validated Successfully"
INVALID_CREDENTIALS = "Validated Failed Invalid APIkey of Toekn"
headers = {
            'Connection': 'keep-alive',
            'Content-Type': 'application/json',
            'Origin': 'https://thor-gpu.e2enetworks.net',
            'Referer': 'https://thor-gpu.e2enetworks.net/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
        }
MANAGED_STORAGE = "managed"
E2E_OBJECT_STORAGE = "e2e_s3"
BUCKET_TYPES = [MANAGED_STORAGE, E2E_OBJECT_STORAGE]
BUCKET_TYPES_HELP = {
    MANAGED_STORAGE: "To Create New Bucket",
    E2E_OBJECT_STORAGE: " To Use Existing Bucket"
}

FREE_USAGE = "free_usage"
PAID_USAGE = "paid_usage"
INSTANCE_TYPE = [FREE_USAGE, PAID_USAGE]
MODEL_TYPES = ['pytorch', 'triton']
S3_ENDPOINT = "objectstore.e2enetworks.net"
hash_code_to_image_id = {
    "QHeMSAyC": 1,
    "skqbtZpd": 2,
    "NynWPeEJ": 3,
    "XEFKbStA": 4,
    "QPSKEFKQ": 5,
    "siGeSJyO": 6,
    "yYkVtTsC": 7,
    "wVOqCnpB": 8,
    "SICGgpoZ": 9,
    "ddUybAyh": 10,
    "yWNKwMlg": 11,
    "WtZaGkNZ": 12,
}



import requests

from typing import Optional
from e2enetworks.constants import BASE_GPU_URL
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.utils import prepare_object


class Teams:
    def __init__(self):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if team:
            client.Default.set_team(team)

    def create(self, team_name):
        payload = {
            "team_name": team_name,
        }
        url = f"{BASE_GPU_URL}teams/"
        req = requests.Request('POST', url, data=payload)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def get(self, team_id):

        if type(team_id) != int:
            raise ValueError(team_id)

        url = f"{BASE_GPU_URL}teams/{team_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, team_id):
        if type(team_id) != int:
            raise ValueError(team_id)

        url = f"{BASE_GPU_URL}teams/{team_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("Teams Class Help")
        print("\t\t================")
        print("\t\tThis class provides functionalities to interact with teams.")
        print("\t\tAvailable methods:")
        print("\t\t1. create(team_name): Creates a new team with the provided team name.")
        print("\t\t2. get(team_id): Retrieves information about a specific team using its ID.")
        print("\t\t3. list(): Lists all teams.")
        print("\t\t4. delete(team_id): Deletes a team with the given ID.")
        print("\t\t5. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tteams = Teams()")
        print("\t\tteams.create('Team Name')")
        print("\t\tteams.get(123)")
        print("\t\tteams.list()")
        print("\t\tteams.delete(123)")
        


import json

import requests

from typing import Optional
from e2enetworks.constants import BASE_GPU_URL, BUCKET_TYPES, MODEL_TYPES, headers, MANAGED_STORAGE
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.cloud.tir.minio_service import MinioService
from e2enetworks.cloud.tir.helpers import get_random_string


class Models:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def create(self, name, model_type, storage_type, bucket_name):
        payload = json.dumps({
            "name": name,
            "model_type": model_type,
            "bucket_name": bucket_name,
            "storage_type": storage_type
        })
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/model/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        return prepare_object(response)

    def push_model(self, model_path, prefix="", model_id=None, model_type="custom"):
        if model_id:
            model = self.get(model_id)
            if model.code == 200:
                access_key = model.data.access_key.access_key
                secret_key = model.data.access_key.secret_key
            else:
                raise ValueError("Invalid model id")
        else:
            model_name = f"model{get_random_string(6)}"
            model = self.create(name=model_name, model_type=model_type, storage_type=MANAGED_STORAGE, bucket_name=None)
            if model.code == 201:
                access_key = model.data.access_key.access_key
                secret_key = model.data.access_key.secret_key

        minio_service = MinioService(access_key=access_key, secret_key=secret_key)
        minio_service.upload_directory_recursive(bucket_name=model.data.bucket.bucket_name,
                                                 source_directory=model_path, prefix=prefix)
        print("Model Pushed Successfully")

    def download_model(self, model_id, local_path, prefix=""):
        if not model_id:
            raise ValueError("model id is mandatory")
        model = self.get(model_id)
        try:
            if model.code == 200:
                access_key = model.data.access_key.access_key
                secret_key = model.data.access_key.secret_key
            else:
                raise ValueError("Invalid model id")
            minio_service = MinioService(access_key=access_key, secret_key=secret_key)
            minio_service.download_model(bucket_name=model.data.bucket.bucket_name, local_path=local_path, prefix=prefix)
            print("Model Download Successfully Successfully")
        except Exception as e:
            print(e)
            raise ValueError("Invalid model id")

    def get(self, model_id):

        if type(model_id) != int:
            raise ValueError(model_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/model/" \
              f"{model_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/model/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, model_id):
        if type(model_id) != int:
            raise ValueError(model_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/model/" \
              f"{model_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("Models Class Help")
        print("\t\t=================")
        print("\t\tThis class provides functionalities to interact with models.")
        print("\t\tAvailable methods:")
        print(
            "\t\t1. __init__(team_id, project_id): Initializes a Models instance with the specified team and project "
            "IDs.")
        print("\t\t2. create(name, storage_type, bucket_name, model_type): Creates a new model with the provided "
              "details.")
        print("\t\t2. push_model(model_path, prefix, model_id, model_type): Creates a new model with the provided "
              "details.")
        print("\t\t3. download_model(model_id, local_path, prefix)")
        print("\t\t4. get(model_id): Retrieves information about a specific model using its ID.")
        print("\t\t5. list(): Lists all models associated with the team and project.")
        print("\t\t6. delete(model_id): Deletes a model with the given ID.")
        print("\t\t7. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tmodels = Models(123, 456)")
        print(f"\t\tmodels.create(name='Test Dataset', bucket_name='dataset-bucket', storage_type={BUCKET_TYPES},"
              f" model_type={MODEL_TYPES})")
        print(f"\t\tmodels.push_model(model_path, prefix='', model_id=None, model_type='custom')")
        print(f"\t\tmodels.download_model(model_id=<model id>, local_path=<path of local directory>,"
              f" prefix=<prefix in teh bucket>)")
        print("\t\tmodels.get(789)")
        print("\t\tmodels.list()")
        print("\t\tmodels.delete(789)")


import json

import requests

from typing import Optional
from e2enetworks.constants import BASE_GPU_URL, INSTANCE_TYPE, headers
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.utils import prepare_object


class Notebooks:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def create(self, name, sku_id, image_id, instance_type, disk_size_in_gb=30, notebook_type="new", notebook_url=""):
        payload = json.dumps({
            "name": name,
            "image_id": image_id,
            "sku_id": sku_id,
            "auto_shutdown_timeout": None,
            "instance_type": instance_type,
            "dataset_id_list": [],
            "disk_size_in_gb": disk_size_in_gb,
            "notebook_type": notebook_type,
            "notebook_url": notebook_url
        })
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        return prepare_object(response)

    def get(self, notebook_id):

        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def stop(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/actions/?action=stop&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def start(self, notebook_id):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/actions/?action=start&"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def upgrade(self, notebook_id, size):
        if type(notebook_id) != int:
            raise ValueError(notebook_id)

        if type(size) != int:
            raise ValueError(notebook_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/notebooks/" \
              f"{notebook_id}/pvc/upgrade/"
        req = requests.Request('PUT', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("\t\tNotebook Class Help")
        print("\t\t=================")
        help_text = f"""
                Notebooks class provides methods to interact with notebooks in a project.

                Available methods:
                1. create(name, sku_id, image_id, instance_type, disk_size_in_gb=30, notebook_type="new",
                 notebook_url=""): Create a new notebook.
                2. get(notebook_id): Get information about a notebook.
                3. list(): List all notebooks in the project.
                4. delete(notebook_id): Delete a notebook.
                5. stop(notebook_id): Stop a running notebook.
                6. start(notebook_id): Start a stopped notebook.
                7. upgrade(notebook_id, size): Upgrade the size of a notebook's PVC.

                Usage:
                notebooks = Notebooks(team_id, project_id)
                notebooks.create(name, sku_id, image_id, instance_type={INSTANCE_TYPE}, disk_size_in_gb=30,
                 notebook_type="new", notebook_url="")
                notebooks.get(notebook_id)
                notebooks.list()
                notebooks.delete(notebook_id)
                notebooks.stop(notebook_id)
                notebooks.start(notebook_id)
                notebooks.upgrade(notebook_id, size)
                """
        print(help_text)



import requests
from e2enetworks.constants import BASE_GPU_URL, hash_code_to_image_id
from e2enetworks.cloud.tir import client
from e2enetworks.cloud.tir.helpers import cpu_plan_short_code, gpu_plan_short_code
from e2enetworks.cloud.tir.utils import prepare_object


class Plans:
    def __init__(self):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

    def list(self, service, image=None):

        if type(service) != str:
            print(f"Service - {service} Should be String")
            return

        if service == "notebook" and type(image) != str:
            print(f"Image ID - {image} Should be String")
            return
        image = hash_code_to_image_id.get(image)
        image = image if image else ""
        url = f"{BASE_GPU_URL}gpu_service/sku/?image_id={image}&service={service}&"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        if response.status_code == 200:
            skus = response.json()["data"]
            for sku in skus["CPU"]:
                if sku.get("is_inventory_available"):
                    print(cpu_plan_short_code(sku))
            for sku in skus["GPU"]:
                if sku.get("is_inventory_available"):
                    print(gpu_plan_short_code(sku))

    def get_skus_list(self, service, image=None):
        image = image if image else ""
        url = f"{BASE_GPU_URL}gpu_service/sku/?image_id={image}&service={service}&"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return response.json()["data"] if response.status_code == 200 else prepare_object(response)

    @staticmethod
    def help():
        print("Sku Class Help")
        print("\t\t================")
        print("\t\tThis class provides functionalities to interact with Plans.")
        print("\t\tAvailable methods:")

        print("\t\t1. list(service, image_id): Lists all Plans for given image_id and service.\n")
        print("\t\t Allowed Services List - ['notebook', 'inference_service', 'pipeline']")
        # Example usages
        print("\t\tExample usages:")
        print("\t\tskus = Plans()")
        print("\t\tskus.list('inference')")



from minio import Minio
from minio.error import S3Error
import os

from e2enetworks.constants import S3_ENDPOINT


class MinioService:

    def __init__(self, access_key, secret_key, endpoint=S3_ENDPOINT):
        self.endpoint = endpoint
        self.access_key = access_key
        self.secret_key = secret_key
        self.client = Minio(endpoint, access_key, secret_key)

    def upload_directory_recursive(self, bucket_name, source_directory, prefix=""):
        for root, dirs, files in os.walk(source_directory):
            for file in files:
                file_path = os.path.join(root, file)
                object_name = os.path.join(prefix, os.path.relpath(file_path, source_directory))
                try:
                    self.client.fput_object(bucket_name, object_name, file_path)
                    print(f"Uploaded {object_name}")
                except S3Error as e:
                    print(f"Error uploading {object_name}: {e}")

    def download_model(self, bucket_name, local_path, prefix=""):
        objects = self.client.list_objects(bucket_name=bucket_name, prefix=prefix, recursive=True)
        for obj in objects:
            object_name = obj.object_name
            try:
                self.client.fget_object(bucket_name, object_name, f"{local_path}/{object_name}")
                print(f"Downloaded: {object_name}")
            except Exception as err:
                print(f"Error downloading {object_name}: {err}")



import string
import random


def get_random_string(N):
    return ''.join(random.choices(string.ascii_lowercase + string.digits, k=N))


def cpu_plan_short_code(sku):
    return f"{sku.get('series')}-{sku.get('sku_type').split('-')[0]}-{sku.get('cpu')}-" \
           f"{sku.get('memory')}-{sku.get('gpu') if sku.get('gpu') else 0}"


def gpu_plan_short_code(sku):
    return f"{sku.get('series')}-{sku.get('sku_type').split('.')[1]}-{sku.get('cpu')}-" \
           f"{sku.get('memory')}-{sku.get('gpu') if sku.get('gpu') else 0}"


def plan_to_sku_id(skus, plan):
    if plan.startswith("CPU"):
        for sku in skus["CPU"]:
            if cpu_plan_short_code(sku) == plan:
                return sku.get("sku_id")
    else:
        for sku in skus["GPU"]:
            if gpu_plan_short_code(sku) == plan:
                return sku.get("sku_id")
    return False



import json

import requests

from typing import Optional
from e2enetworks.constants import BASE_GPU_URL, headers
from e2enetworks.cloud.tir import client, Plans
from e2enetworks.cloud.tir.utils import prepare_object
from e2enetworks.cloud.tir.helpers import plan_to_sku_id

containers = {
    "llma": "amole2e/meta-7b",
    "llma_eos": "amole2e/meta-7b-chat",
    "codellama": "aimle2e/codellama",
    "codellama_eos": "aimle2e/codellama-eos",
    "mpt": "aimle2e/mpt-7b-chat:hf-v3",
    "mpt_eos": "aimle2e/mpt-7b-chat:eos-v2",
    "stable_diffusion": "aimle2e/stable-diffusion-2-1:hf-v1",
    "stable_diffusion_eos": "aimle2e/stable-diffusion-2-1:eos-v1"
}


class EndPoints:
    def __init__(
            self,
            team: Optional[str] = "",
            project: Optional[str] = ""
    ):
        client_not_ready = (
            "Client is not ready. Please initiate client by:"
            "\n- Using e2enetworks.cloud.tir.init(...)"
        )
        if not client.Default.ready():
            raise ValueError(client_not_ready)

        if project:
            client.Default.set_project(project)

        if team:
            client.Default.set_team(team)

    def get_container_name(self, container_name, model_id, framework):
        if framework == "custom":
            return container_name
        if model_id:
            return containers[framework+'_eos'] if framework+'_eos' in containers else None
        else:
            return containers[framework] if framework in containers else None

    def create(self, endpoint_name, framework, plan, model_id=None, container_name=None, container_type="public",
               replicas=1, disc_size=10, model_path="", env_variables=[], mount_path="", registry_endpoint="",
               auth_type="pass", username="", password="", docker_config=""):
        if not endpoint_name:
            raise ValueError(endpoint_name)
        if not framework:
            raise ValueError(framework)
        if not plan:
            raise ValueError(plan)

        skus = Plans().get_skus_list("inference_service")
        sku_id = plan_to_sku_id(skus=skus, plan=plan)

        if not sku_id:
            raise ValueError(plan)

        container_name = self.get_container_name(container_name=container_name, model_id=model_id, framework=framework)

        if framework not in ["pytorch", "triton"] and not container_name:
            raise ValueError(container_name)

        payload = json.dumps({
            "name": endpoint_name or "",
            "path": model_path,
            "custom_endpoint_details": {
                "container": {
                    "container_name": container_name,
                    "container_type": container_type,
                    "private_image_details": {
                        "registry_endpoint": registry_endpoint,
                        "auth_type": auth_type,
                        "password_auth_details": {
                            "username": username,
                            "password": password
                        },
                        "docker_config": {
                            "code": docker_config
                        }
                    }
                },
                "resource_details": {
                    "disk_size": disc_size,
                    "mount_path": mount_path,
                    "env_variables": env_variables
                },
            },
            "model_id": model_id,
            "sku_id": sku_id,
            "replica": replicas or 1,
            "framework": framework
        })
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/?" \
              f"apikey={client.Default.api_key()}"
        response = requests.post(url=url, headers=headers, data=payload)
        print(f"To check the Inference Status and logs, PLease visit "
              f"https://gpu-notebooks.e2enetworks.com/projects/{client.Default.project()}/model-endpoints")
        return prepare_object(response)

    def get(self, endpoint_id):

        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def logs(self, endpoint_id):

        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/logs/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def start(self, endpoint_id):
        payload = json.dumps({
            "action": "start"
        })
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def stop(self, endpoint_id):
        payload = json.dumps({
            "action": "stop"
        })
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)
        headers['Authorization'] = f'Bearer {client.Default.access_token()}'
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('PUT', url=url, data=payload, headers=headers)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def list(self):
        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/"
        req = requests.Request('GET', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    def delete(self, endpoint_id):
        if type(endpoint_id) != int:
            raise ValueError(endpoint_id)

        url = f"{BASE_GPU_URL}teams/{client.Default.team()}/projects/{client.Default.project()}/serving/inference/" \
              f"{endpoint_id}/"
        req = requests.Request('DELETE', url)
        response = client.Default.make_request(req)
        return prepare_object(response)

    @staticmethod
    def help():
        print("EndPoint Class Help")
        print("\t\t=================")
        print("\t\tThis class provides functionalities to interact with EndPoint.")
        print("\t\tAvailable methods:")
        print("\t\t1. __init__(team, project): Initializes an EndPoints instance with the specified team and "
              "project IDs.")
        print("\t\t2. create(endpoint_name, framework, plan, container_name, container_type, model_id, replicas=1, "
              "disc_size=10, model_path="", env_variables=[], mount_path="", registry_endpoint="", "
              "auth_type='pass', username="", password="", docker_config=""): "
              "Creates an endpoint with the provided details.")
        print("\t\t3. get(endpoint_id): Retrieves information about a specific endpoint using its ID.")
        print("\t\t3. logs(endpoint_id): Retrieves logs of a specific endpoint using its ID.")
        print("\t\t3. stop(endpoint_id): Stops a specific endpoint using its ID.")
        print("\t\t3. start(endpoint_id): Starts a specific endpoint using its ID.")
        print("\t\t4. list(): Lists all endpoints associated with the team and project.")
        print("\t\t5. delete(endpoint_id): Deletes an endpoint with the given ID.")
        print("\t\t8. help(): Displays this help message.")

        # Example usages
        print("\t\tExample usages:")
        print("\t\tendpoints = EndPoints(123, 456)")
        print("\t\tendpoints.create("
              "\n\t\t\t\tendpoint_name(required):String => 'Name of Endpoint'",
              "\n\t\t\t\tframework(required):String => '['triton', 'pytorch', 'llma', 'stable_diffusion', 'mpt,"
              "\n\t\t\t\t\t'codellama', 'custom']'",
              "\n\t\t\t\tplan(required):String=> Plans Can be listed using tir.Plans Apis",
              "\n\t\t\t\tcontainer_type(optional):String=> Default value is public and "
              "\n\t\t\t\t\tallowed values are [public, private]",
              "\n\t\t\t\tmodel_id:Integer=> Required in case of Framework type=[triton, pytorch] and "
              "\n\t\t\t\t\tif model is stored in EOS",
              "\n\t\t\t\tcontainer_name(optional):String=> Docker Container Image Name required in case of Custom "
              "\n\t\t\t\tContainer Only",
              "\n\t\t\t\treplicas(optional):Integer=> Default value id 1",
              "\n\t\t\t\tdisc_size(optional):Integer=> Default value id 10Gb",
              "\n\t\t\t\tmodel_path(optional):String=> Path of EOS bucket where the model is stored",
              "\n\t\t\t\tenv_variables(optional):List=> Env variables can be passed as "
              "\n\t\t\t\t\t[{ 'key': '', 'value': '/mnt/models'}]"
              "\n\t\t\t\tmount_path(optional):String=> Default value is '/mnt/models'"
              "\n\t\t\t\tregistry_endpoint(optional):String=> Required in Case of container_type=private"
              "\n\t\t\t\tauth_type(optional):String=> Required in case of container_type=private, "
              "\n\t\t\t\t\tAllowed Values are ['pass', 'docker'] "
              "\n\t\t\t\t\tDefault Value is pass'"
              "\n\t\t\t\tusername(optional):String=> Required in case of container_type=private and auth_type=pass"
              "\n\t\t\t\tusername(optional):String=> Required in case of container_type=private and auth_type=pass"
              "\n\t\t\t\tdocker_config(optional):String=> Required in case of container_type=private and auth_type=docker")
        print("\t\tendpoints.get(789)")
        print("\t\tendpoints.logs(789)")
        print("\t\tendpoints.stop(789)")
        print("\t\tendpoints.start(789)")
        print("\t\tendpoints.list()")
        print("\t\tendpoints.delete(789)")
